import { Link, useRouteData } from "solid-app-router";
import { createEffect, createResource, For, Show } from "solid-js";
import server from "solid-start/server";
import { login, serveLogout, authData } from "~/server/auth";
import { composeRouteData, useServer, withRC } from "~/util";
import { serveTodos } from "~/server/todos";
import { useRefreshy } from "~/util/refreshy";

export const routeData = () => {
  const [refreshy] = useRefreshy();

  return composeRouteData({
    auth: authData(),
    todos: () => createResource(refreshy(() => serveTodos())),
  })();
};

type Todo = { title: string };

export default function Home() {
  const [data, refetch] = useRouteData();

  const onLogin = useServer(
    server(withRC((rc) => login(rc, "admin@example.com", "d1r3ctu5", false))),
    refetch
  );

  const onLogout = useServer(serveLogout, refetch);

  return (
    <main class="text-center mx-auto container text-gray-700">
      <div class="my-7 flex items-center gap-4 justify-end">
        <Show when={data?.auth?.username}>
          <div>Hey {data.auth.username}</div>
        </Show>

        <Show
          when={data?.auth?.username}
          fallback={
            <>
              <button class="btn btn-sm" onClick={onLogin}>
                Login instant
              </button>
              <Link class="btn btn-sm" href="/login">
                Login form
              </Link>
            </>
          }
        >
          <button class="btn btn-sm" onClick={onLogout}>
            Logout
          </button>
        </Show>

        <Link class="btn btn-sm" href="/jwtTest">
          JwtTest
        </Link>
      </div>

      <Show
        when={data?.auth?.username}
        fallback={<div class="text-xl">Login first</div>}
      >
        <Show when={data?.todos}>
          <div class="text-xl">Directus data:</div>
          <div class="mx-auto flex flex-col gap-3">
            <For each={data?.todos}>
              {(item: Todo) => (
                <div class="card bg-base-100 shadow-xl">
                  <div class="card-body">{item.title}</div>
                </div>
              )}
            </For>
          </div>
        </Show>
      </Show>
    </main>
  );
}
