import { gql, OperationResult } from "urql";
import { getSession, isLoggedIn, logout } from "./auth";
import server from "solid-start/server";
import { awaitJson, RC, withRC } from "../util";
import { createDirectusClient } from "./directus";

export const getTodos = async function (rc: RC) {
  const session = await getSession(rc);

  if (!isLoggedIn(session)) {
    return [];
  }

  let result: OperationResult;
  let authError = false;

  try {
    const client = createDirectusClient(session, () => {
      authError = true;
    });
    result = await client
      .query(
        gql`
          query {
            Todos {
              title
            }
          }
        `
      )
      .toPromise();

    if (result.error) {
      console.dir(result.error);
      throw new Error(result.error.response);
    }
  } catch (err) {
    console.error(err);
    return [];
  }

  if (authError) {
    console.log("logout");
    return await logout(rc, "/login");
  }

  if (!result.data) return [];

  return result.data.Todos;
};

export const serveTodos = server(withRC((rc) => getTodos(rc)));
