import {
  createContext,
  createSignal,
  FlowComponent,
  useContext,
} from "solid-js";
import { isServer } from "solid-js/web";
import { Middleware, MiddlewareFn } from "solid-start/entry-server";

const refresh_ = async function (
  ctx: ReturnType<typeof createRefreshyContext>
) {
  const time = new Date().getTime();

  if (ctx.lastRefresh + ctx.ttl * 1000 > time) {
    return;
  }

  ctx.lastRefresh = time;

  ctx.refreshPromise = (async () => {
    const result = await ctx.refresh(ctx);
    ctx.refreshResult[1](result);
    ctx.refreshPromise = null;
  })();

  await ctx.refreshPromise;
};

export type RefreshyContext = ReturnType<typeof createRefreshyContext>;

const createRefreshyContext = function (
  refresh: Function,
  ttl = 60,
  refreshSoon = false
) {
  const refreshResult = createSignal();
  const refreshy = <T extends (...args: any[]) => Promise<any>>(fetcher: T) => {
    if (isServer) {
      return fetcher;
    }

    return async (...args: Parameters<T>) => {
      if (ctx.refreshPromise) {
        await ctx.refreshPromise;
      } else {
        await refresh_(ctx);
      }

      return await fetcher(...args);
    };
  };

  const ctx = {
    ttl,
    refresh,
    refreshResult,
    refreshy,
    lastRefresh: isServer || refreshSoon ? 0 : new Date().getTime(),
    refreshPromise: null as Promise<any> | null,
  };

  return ctx;
};
const FetcherContext = createContext(
  {} as ReturnType<typeof createRefreshyContext>
);
export type RefreshFunction = (ctx: RefreshyContext) => any;
export const RefreshyProvider: FlowComponent<{
  ttl?: number;
  refresh: RefreshFunction;
  refreshSoon?: boolean;
}> = (props) => {
  const ctx = createRefreshyContext(
    props.refresh,
    props.ttl,
    props.refreshSoon
  );

  return (
    <FetcherContext.Provider value={ctx}>
      {props.children}
    </FetcherContext.Provider>
  );
};

export const createRefreshMiddleware = (
  refresh: (request: Request, responseHeaders: Headers) => Promise<any>
) => {
  return function ({ forward }) {
    return async (ctx) => {
      await refresh(ctx.request, ctx.responseHeaders);

      return await forward(ctx);
    };
  } as Middleware;
};

export const useRefreshy = function () {
  const ctx = useContext(FetcherContext);
  const refreshy = ctx.refreshy;
  const signal = ctx.refreshResult[0];

  return [refreshy, signal] as [typeof refreshy, typeof signal];
};
